package ac.essex.ooechs.imaging.commons.apps.training.strategies;

import ac.essex.ooechs.imaging.commons.util.Region;
import ac.essex.ooechs.imaging.commons.Pixel;

import java.util.Vector;
import java.awt.image.BufferedImage;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 23-Oct-2006
 * @version 1.0
 */
public class SquareCroppingStrategy implements CroppingStrategy {


    public String getName() {
        return "Square Cropper";
    }

    public String getDescription() {
        return "Click on the bottom left and top right of the object to crop.";
    }

    public int countNodes() {
        return 2;
    }

    public double getAspectRatio() {
        return 1.0;
    }

    public Region getCropRegion(BufferedImage image, Vector<Pixel> nodes) throws StrategyException {

        if (nodes.size() != countNodes()) {
            throw new StrategyException("Cannot crop, I need two positions first!");
        }

        Pixel point1 = nodes.elementAt(0);
        Pixel point2 = nodes.elementAt(1);

        return getCroppingRegion(point1, point2);

    }

    public Region getCroppingRegion(Pixel point1, Pixel point2) {

        // put eyes in correct order
        Pixel leftPoint, rightPoint;
        Pixel topPoint, bottomPoint;

        if (point1.x < point2.x) {
            leftPoint = point1;
            rightPoint = point2;
        } else {
            leftPoint = point2;
            rightPoint = point1;
        }

        if (point1.y < point2.y) {
            topPoint = point1;
            bottomPoint = point2;
        } else {
            topPoint = point2;
            bottomPoint = point1;
        }

        // find the width and height
        int width = rightPoint.x - leftPoint.x;
        int height = bottomPoint.y - topPoint.y;

        int size = Math.max(width, height);

        int left = leftPoint.x - ((size - width) / 2);
        int top = topPoint.y - ((size - height) / 2);

        return new Region((int) left, (int) top, (int) size, (int) size);

    }

}
