package ac.essex.ooechs.imaging.commons.corner;

import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.Pixel;
import ac.essex.ooechs.imaging.commons.util.panels.ImageFrame;
import ac.essex.ooechs.imaging.commons.edge.CannyEdgeDetector;

import java.util.Vector;
import java.awt.*;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 03-Jun-2008
 * @version 1.0
 */
public class CornerDetector {

    static int halfWindowSize = 5;

    public static void main(String[] args) throws Exception {

        // take an image
        PixelLoader p = new PixelLoader("/home/ooechs/susan/phantom_shapes.jpg");

        // perform edge detection on it
        CannyEdgeDetector detector = new CannyEdgeDetector();
        detector.setSourceImage(p);
        detector.process();

        // get the edge image
        PixelLoader edges = new PixelLoader(detector.getEdgesImage());
        //PixelLoader edges = new PixelLoader("/home/ooechs/Desktop/test.bmp");

        int angleSteps = 8;
        final double MAX_ANGLE = Math.PI * 2;
        double angleStep = MAX_ANGLE / angleSteps;

        int c = 0;
        for (double a = 0; a < MAX_ANGLE; a+= angleStep) {
            System.out.println(c + ", " + a);
            c++;
        }

        Vector<Pixel> corners = new Vector<Pixel>();

        System.out.println("Edges.getHeight() " + edges.getHeight());
        System.out.println("Edges.getWidth() " + edges.getWidth());

        // perform a series of local hough transforms to find simple lines
        for (int y = halfWindowSize; y < edges.getHeight() - halfWindowSize; y++) {
            for (int x = halfWindowSize; x < edges.getWidth() - halfWindowSize; x++) {

                int[] angleCount = new int[angleSteps];

                for (int dy = -halfWindowSize; dy < halfWindowSize; dy++) {
                    for (int dx = -halfWindowSize; dx < halfWindowSize; dx++) {

                        if (edges.getGreyValue(x + dx, y + dy) > 0) {

                            // do a very simple hough transform looking for simple lines
                            double angle;

                            if (dx != 0) {
                                // find the angle
                                double m = dy / (double) dx;
                                angle = Math.atan(m);
                            } else {
                                // set to 90 degrees
                                if (dy < 0) {
                                    angle = Math.PI/2;
                                } else {
                                    angle = -Math.PI/2;
                                }
                            }

                            if (dx == 0 & dy == 0) {
                                // have it hist towards all lines
                                for (int i = 0; i < angleCount.length; i++) {
                                    angleCount[i]++;
                                }
                            } else {
                                int angleSlot = ((int) (angle / angleStep)) + (angleSteps/2);
                                angleCount[angleSlot]++;
                            }

                        }

                    }
                }

                int lineCount = 0;

                for (int i = 0; i < angleCount.length; i++) {
                    int count = angleCount[i];
                    //System.out.print(i + ", " + hist);
                    if (count >= halfWindowSize) {
                        lineCount++;
                        // TODO: Figure out which are part of the same line - they are not
                        //corners.
                        //System.out.println("*");
                    } else {
                        //System.out.println();
                    }

                }

                // if more than one line, designate center as being a corner
                if (lineCount > 1) {

                    //System.out.println("This is a corner");
                    corners.add(new Pixel(x, y));
                } else {
                    //System.out.println("This is not a corner");
                }

            }
        }

        // draw the corners on the edge image
        Graphics g = edges.getBufferedImage().getGraphics();
        for (int i = 0; i < corners.size(); i++) {
            Pixel corner = corners.elementAt(i);
            int windowSize = halfWindowSize * 2;
            g.setColor(Color.RED);
            g.drawRect(corner.x - halfWindowSize, corner.y - halfWindowSize, windowSize, windowSize);
        }

        new ImageFrame(edges);

    }
}
