package ac.essex.ooechs.imaging.commons.edge;

import ac.essex.ooechs.imaging.commons.util.panels.ImagePanel;
import ac.essex.ooechs.imaging.commons.util.panels.FileTree;
import ac.essex.ooechs.imaging.commons.util.ImageFilenameFilter;
import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.ConvolutionMatrix;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.io.File;
import java.awt.image.BufferedImage;
import java.awt.*;

public class EdgeDetectionGUI extends JFrame {

    protected ImagePanel p;
    protected JButton load, segment;
    protected JSlider thresholdSlider;

    public EdgeDetectionGUI(File file) {
        super("Edge Detection");

        p = new ImagePanel();
        //p.setDisplayCentered(true);

        FileTree f = new FileTree(file, new ImageFilenameFilter()) {

            /**
             * Called when a file is selected in the tree
             */
            public void onSelectFile(File f) {
                loadImage(f);
            }

        };


        JToolBar bar = new JToolBar();

        bar.add(new JLabel("Edge Detection: Sobel, Threshold:"));
        thresholdSlider = new JSlider(0, 512);
        bar.add(thresholdSlider);
        thresholdSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                if (!thresholdSlider.getValueIsAdjusting()) {
                    segment();
                }
            }
        });

        getContentPane().add(bar, BorderLayout.NORTH);
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, new JScrollPane(f), new JScrollPane(p));
        getContentPane().add(splitPane, BorderLayout.CENTER);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setSize(640, 480);
        setVisible(true);
    }

    protected void loadImage(File f) {
        if (f != null) {

                this.f = f;

                segment();

        }
    }

    File f = null;

    protected void segment() {
        new Thread() {
            public void run() {

                if (f == null) return;

            try {
                p.setImage(new PixelLoader(f));
            } catch (Exception err) {
                err.printStackTrace();
                return;
            }

                int threshold = thresholdSlider.getValue();

                BufferedImage image = p.getImage();

                PixelLoader pl = new PixelLoader(image);

                ConvolutionMatrix HSOBEL = new ConvolutionMatrix(ConvolutionMatrix.HORIZONTAL_SOBEL);
                ConvolutionMatrix VSOBEL = new ConvolutionMatrix(ConvolutionMatrix.VERTICAL_SOBEL);

                double[][] data = new double[pl.getWidth()][pl.getHeight()];

                for (int y = 0; y < image.getHeight(); y += 1) {
                    for (int x = 0; x < image.getWidth(); x += 1) {

                        double gx = pl.getConvolved(x, y, HSOBEL);
                        double gy = pl.getConvolved(x, y, VSOBEL);

                        double G = Math.abs(gx) + Math.abs(gy);

                        data[x][y] = G;

                    }
                }

                int neighbourhoodSize = 0;

                for (int y = neighbourhoodSize; y < image.getHeight() - neighbourhoodSize; y++) {
                    loop: for (int x = neighbourhoodSize; x < image.getWidth() - neighbourhoodSize; x++) {

                        double G = data[x][y];

                        if (G > threshold) {

                        // Check that this peak is indeed the local maxima
                           for (int dx = -neighbourhoodSize; dx <= neighbourhoodSize; dx++) {
                               for (int dy = -neighbourhoodSize; dy <= neighbourhoodSize; dy++) {
                                   if (data[dx + x][dy + y] > G) {
                                       // found a bigger point nearby, skip
                                       continue loop;
                                   }
                               }
                           }

                            image.setRGB(x, y, Color.RED.getRGB());

                        }
                    }
                }

                p.repaint();

            }
        }.start();
    }

    public static void main(String args[]) {
        File directory = new File("/home/ooechs/Desktop/Documents/Papers/Pipelines/data");
        new EdgeDetectionGUI(directory);
    }

}
