package ac.essex.ooechs.imaging.commons.matching;

import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.Pixel;
import ac.essex.ooechs.imaging.commons.Resizer;
import ac.essex.ooechs.imaging.commons.util.ImageFilenameFilter;
import ac.essex.ooechs.imaging.commons.util.panels.ImagePanel;
import ac.essex.ooechs.imaging.commons.util.panels.ScalingImageFrame;

import javax.swing.*;
import java.io.File;
import java.util.Vector;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.RescaleOp;

/**
 * Created by IntelliJ IDEA.
 * User: Olly
 * Date: 30-Nov-2008
 * Time: 09:09:56
 * To change this template use File | Settings | File Templates.
 */
public class MatchingGUI extends JFrame {

    protected Vector<File> files;
    protected int cursor = 0;
    protected PixelLoader currentImage;
    protected String currentFilename;
    protected File currentDirectory;
    protected PointSelectionPanel panel;

    protected Vector<MatchImage> matchImages;

    public static void main(String[] args) {
        new MatchingGUI(new File("c:\\campusplans\\"));
    }

    public MatchingGUI(File directory) {

        matchImages = new Vector<MatchImage>();

        getFiles(directory);

        addKeyListener(new KeyAdapter() {

            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_SPACE) {
                    next();
                }
                if (e.getKeyCode() == KeyEvent.VK_UP) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.y--;
                        panel.repaint();
                    }
                }
                if (e.getKeyCode() == KeyEvent.VK_DOWN) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.y++;
                        panel.repaint();
                    }
                }
                if (e.getKeyCode() == KeyEvent.VK_LEFT) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.x--;
                        panel.repaint();
                    }
                }
                if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.x++;
                        panel.repaint();
                    }
                }

            }

        });

        panel = new PointSelectionPanel();
        getContentPane().add(new JScrollPane(panel));

        setTitle("Image Matching GUI");
        setSize(640, 480);
        setVisible(true);
        setLocationRelativeTo(null);

        next();


    }

    public void getFiles(File directory) {
        this.currentDirectory = directory;
        files = new Vector<File>(50);
        File[] fileList = directory.listFiles(new ImageFilenameFilter());
        for (int i = 0; i < fileList.length; i++) {
            File file = fileList[i];
            files.add(file);
        }
    }

    public boolean next() {
        try {
            if (cursor < files.size()) {
                currentImage = new PixelLoader(files.elementAt(cursor));
                currentFilename = currentImage.getFilename();
                panel.setImage(currentImage);
                cursor++;
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    class PointSelectionPanel extends ImagePanel {

        public Point currentPixel, pointA, pointB;

        int mode = 0;

        public PointSelectionPanel() {

            addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent e) {

                    if (mode == 0) {
                        pointA = new Point(getX(e), getY(e));
                        currentPixel = pointA;
                        mode++;
                        repaint();
                        return;
                    }

                    if (mode == 1) {
                        pointB = new Point(getX(e), getY(e));
                        mode++;
                        repaint();
                        currentPixel = pointB;
                        return;
                    }

                    if (mode == 2) {
                        matchImages.add(new MatchImage(currentImage, pointA, pointB));
                        if (!next()) {
                            process();
                        }
                    }

                }
            });

            addMouseWheelListener(new MouseWheelListener() {
                public void mouseWheelMoved(MouseWheelEvent e) {
                    if (e.getUnitsToScroll() > 0) {
                        zoomOut();
                    } else {
                        zoomIn();
                    }
                    repaint();
                }
            });

            addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseMoved(MouseEvent e) {

                    if (mode > 1) {
                        repaint();
                    }

                }
            });

        }

        public void setImage(PixelLoader image) {
            super.setImage(image);
            mode = 0;
            pointA = null;
            pointB = null;
        }

        public void paintComponent(Graphics g) {

            super.paintComponent(g);

            if (pointA != null) paintNode(pointA, g);
            if (pointB != null) paintNode(pointB, g);

        }

        public void paintNode(Point node, Graphics g) {
            int crosshairSize = 10;
            g.setColor(Color.RED);
            drawLine(g, (int) node.x, (int) node.y - crosshairSize, (int) node.x, (int) node.y + crosshairSize);
            drawLine(g, (int) node.x + crosshairSize, (int) node.y, (int) node.x - crosshairSize, (int) node.y);
        }

    }




    class MatchImage {

        public PixelLoader image;
        public Point pointA, pointB;

        public double dx, dy;

        public double scaleX = 1, scaleY = 1;

        // translation
        public double tX, tY;

        public int newWidth, newHeight;

        public MatchImage(PixelLoader image, Point pointA, Point pointB) {
            this.image = image;
            this.pointB = pointB;
            this.pointA = pointA;

            dx = Math.abs(pointB.x - pointA.x);
            dy = Math.abs(pointB.y - pointA.y);

        }

        public double getDist() {
            return Math.sqrt((dx * dx) + (dy * dy));
        }

        public void scale() {
            BufferedImage original = image.getBufferedImage();
            newWidth = (int) (original.getWidth() * scaleX);
            newHeight = (int) (original.getHeight() * scaleY);
        }

        public BufferedImage getMatchImage() {

            BufferedImage original = image.getBufferedImage();

            // scale the original
            BufferedImage newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_INT_ARGB);
            Graphics g = newImage.getGraphics();
            g.drawImage(original, 0, 0, newWidth, newHeight, null);

            return newImage;

        }

    }

    public BufferedImage generateMatchImage() {
        
        double minX = 0;
        double minY = 0;
        double maxX = 0;
        double maxY = 0;

        for (int i = 0; i < matchImages.size(); i++) {
            MatchImage matchImage = matchImages.elementAt(i);
            matchImage.scale();
            if (matchImage.tX < minX) minX = matchImage.tX;
            if (matchImage.tY < minY) minY = matchImage.tY;
            double mX = matchImage.tX + matchImage.newWidth;
            double mY = matchImage.tY + matchImage.newHeight;
            if (mX > maxX) maxX = mX;
            if (mY > maxY) maxY = mY;
        }

        int width = (int) (maxX - minX);
        int height = (int) (maxY - minY);

        float[] scales = { 1f, 1f, 1f, 0.5f };
        float[] offsets = new float[4];
        RescaleOp rop = new RescaleOp(scales, offsets, null);


        BufferedImage matchImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2d = (Graphics2D) matchImage.getGraphics();

        for (int i = 0; i < matchImages.size(); i++) {
            MatchImage image = matchImages.elementAt(i);
            g2d.drawImage(image.getMatchImage(), rop, (int) (minX + image.tX), (int) (minY + image.tY));
        }

        return matchImage;

    }

    public void process() {

        // find the image with the smallest dist
        double smallestDist = Double.MAX_VALUE;
        MatchImage smallest = null;
        for (int i = 0; i < matchImages.size(); i++) {
            MatchImage matchImage = matchImages.elementAt(i);
            double dist = matchImage.getDist();
            if (dist < smallestDist) {
                smallestDist = dist;
                smallest = matchImage;
            }
        }

        // smallest is the image that we won't change, so everything else needs to be fitted around it
        for (int i = 0; i < matchImages.size(); i++) {
            MatchImage matchImage = matchImages.elementAt(i);
            if (matchImage != smallest) {

                matchImage.scaleX = matchImage.dx / smallest.dx;
                matchImage.scaleY = matchImage.dy / smallest.dy;

                // calculate what happens to the points x and y
                matchImage.pointA.x /= matchImage.scaleX;
                matchImage.pointA.y /= matchImage.scaleY;

                matchImage.pointB.x /= matchImage.scaleX;
                matchImage.pointB.y /= matchImage.scaleY;

                // now find the translation so that all points are the same
                matchImage.tX = smallest.pointA.x - matchImage.pointA.x;
                matchImage.tY = smallest.pointA.y - matchImage.pointA.y;

            }
        }

        BufferedImage img = generateMatchImage();
        new ScalingImageFrame(img);

    }

}
