package ac.essex.ooechs.imaging.commons.thresholding;

/*
 * Mixture Modeling algorithm
 *
 * Copyright (c) 2003 by Christopher Mei (christopher.mei@sophia.inria.fr)
 *                    and Maxime Dauphin
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//01628 518000 ask fir jprobe support


public class GrayLevelClassMixtureModelling {

    public int[] histogram;
    /**
     * The index must vary between 1 and 253
     * C1 : [0;index]
     * C2 : [index+1; 255]
     */
    private int threshold;
    private float mu1, mu2;
    private float sigma2_1, sigma2_2;
    private float mult1, mult2;
    private float twoVariance1, twoVariance2;
    private int total1, total2;

    private int INDEX_MIN = 1;
    private int INDEX_MAX = 253;
    public static final int MAX = 255;

    public GrayLevelClassMixtureModelling(int N, int[] hist) {
        histogram = hist;
        threshold = INDEX_MIN - 1;
    }

    public boolean incrementThreshold() {
        threshold++;

        if (!(threshold <= INDEX_MAX))
            return false;

        setValues();
        return true;
    }

    private void setValues() {

        mu1 = 0;
        mu2 = 0;
        sigma2_1 = 0;
        sigma2_2 = 0;
        total1 = 0;
        total2 = 0;

        // calculate average for class 0
        for (int i = 0; i <= threshold; i++) {
            total1 += histogram[i];
            mu1 += i * histogram[i];
        }

        // calculate the average for class 1
        for (int i = threshold + 1; i <= MAX; i++) {
            total2 += histogram[i];
            mu2 += i * histogram[i];
        }

        if (total1 > 0) mu1 /= (float) total1;
        if (total2 > 0) mu2 /= (float) total2;

        if (mu1 != 0) {

            for (int i = 0; i <= threshold; i++) {
                sigma2_1 += histogram[i] * Math.pow(i - mu1, 2);
            }

            sigma2_1 /= (float) total1;

            mult1 = getAverage((int) mu1);

            twoVariance1 = 2 * sigma2_1;
        }

        if (mu2 != 0) {

            for (int i = threshold + 1; i <= MAX; i++) {
                sigma2_2 += histogram[i] * Math.pow(i - mu2, 2);
            }

            sigma2_2 /= (float) total2;

            mult2 = getAverage((int) mu2);

            twoVariance2 = 2 * sigma2_2;
        }

    }

    /**
     * Gets the average of the point and its two neighbours
     */
    private float getAverage(int index) {
        float sum = histogram[index];
        int n = 1;
        if (index - 1 >= 0) {
            sum += histogram[index - 1];
            n++;
        }
        if (index + 1 < MAX) {
            sum += histogram[index + 1];
            n++;
        }
        return sum / n;
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();

        ret.append("Index : " + threshold + "\n");
        ret.append("Mu1 : " + mu1 + " ");
        ret.append("Mu2 : " + mu2 + "\n");
        ret.append("Cardinal1 : " + total1 + " ");
        ret.append("Cardinal2 : " + total2 + "\n");
        ret.append("Variance1 : " + sigma2_1 + " ");
        ret.append("Variance2 : " + sigma2_2 + "\n");

        return ret.toString();
    }


    public float getMu1() {
        return mu1;
    }

    public float getMu2() {
        return mu2;
    }

    public float getVariance1() {
        return sigma2_1;
    }

    public float getVariance2() {
        return sigma2_2;
    }

    public float getTotal1() {
        return total1;
    }

    public float getTotal2() {
        return total2;
    }

    public int getThreshold() {
        return threshold;
    }

    public void setThreshold(int threshold) {
        this.threshold = threshold;
        setValues();
    }

    public final float gamma1(int i) {
        if (sigma2_1 == 0) return 0;
        return (float) (mult1 * Math.exp(-(Math.pow((float) i - mu1, 2)) / twoVariance1));
    }

    public final float gamma2(int i) {
        if (sigma2_2 == 0) return 0;
        return (float) (mult2 * Math.exp(-(Math.pow((float) i - mu2, 2)) / twoVariance2));
    }

    public float gamma(int i) {
        return gamma1(i) + gamma2(i);
    }

    public float gammaDifference(int i) {
        return gamma1(i) - gamma2(i);
    }

}



