package ac.essex.ooechs.imaging.commons.window.TheMondayProject.trends;

/**
 * Represents a trend at which a feature is likely to be relatively constant.
 *
 * Can also encode the logic "at some point between these two points the value
 * is at least this value.
 *
 * @author Olly Oechsle, University of Essex, Date: 14-Apr-2008
 * @version 1.0
 */
public class RangeTrend extends Trend {

  /**
     * The position at which the change takes place
     */
    public int x;

    public int s;

    public double minValue, maxValue;

    /**
     * Encodes the point at which a feature changes
     */
    public RangeTrend(int x, int s, double minValue, double maxValue) {

        this.x = x;
        this.s = s;

        this.minValue = minValue;
        this.maxValue = maxValue;

    }

    public boolean matches(double[] data) {
        try {

            for (int dx = 0; dx < s; dx++) {
                if (data[x + dx] < minValue || data[x + dx] > maxValue) return false;
            }

            return true;

        } catch (ArrayIndexOutOfBoundsException e) {
            return false;
        }

    }
    

}
